@@ -13,6 +13,7 @@ use crate::module::{FlatpakBuildOptions, FlatpakModule, FlatpakModuleItem};
 #[derive(Deserialize)]
 #[derive(Serialize)]
 #[derive(Debug)]
+#[derive(Hash)]
 #[derive(Default)]
 #[serde(rename_all = "kebab-case")]
 #[serde(default)]
@@ -220,26 +221,23 @@ impl FlatpakApplication {
             return Err(format!("{} is not a file.", path));
         }
 
-        if FlatpakApplication::file_path_matches(&file_path.to_str().unwrap()) {
-            let manifest_content = match fs::read_to_string(file_path) {
-                Ok(content) => content,
-                Err(e) => {
-                    return Err(format!(
-                        "Could not read file {}: {}!",
-                        file_path.to_str().unwrap(),
-                        e
-                    ))
-                }
-            };
+        let manifest_format = match FlatpakManifestFormat::from_path(&path) {
+            Some(f) => f,
+            None => return Err(format!("{} is not a Flatpak application manifest.", path)),
+        };
 
-            let manifest = match FlatpakApplication::parse(&path, &manifest_content) {
-                Ok(m) => m,
-                Err(e) => return Err(format!("Failed to parse Flatpak manifest at {}: {}", path, e)),
-            };
-
-            return Ok(manifest);
-        } else {
-            return Err(format!("{} is not a Flatpak manifest.", path));
+        let manifest_content = match fs::read_to_string(file_path) {
+            Ok(content) => content,
+            Err(e) => {
+                return Err(format!("Could not read application manifest at {}: {}", &path, e));
+            }
+        };
+        match FlatpakApplication::parse(manifest_format, &manifest_content) {
+            Ok(m) => Ok(m),
+            Err(e) => Err(format!(
+                "Failed to parse Flatpak application manifest at {}: {}",
+                path, e
+            )),
         }
     }
 
@@ -251,27 +249,31 @@ impl FlatpakApplication {
         crate::filename::is_reverse_dns(&path)
     }
 
-    pub fn parse(manifest_path: &str, manifest_content: &str) -> Result<FlatpakApplication, String> {
-        let mut flatpak_manifest: FlatpakApplication = FlatpakApplication::default();
-
-        if crate::filename::is_yaml(&manifest_path) {
-            flatpak_manifest = match serde_yaml::from_str(&manifest_content) {
+    pub fn parse(format: FlatpakManifestFormat, manifest_content: &str) -> Result<FlatpakApplication, String> {
+        let mut flatpak_manifest: FlatpakApplication = match &format {
+            FlatpakManifestFormat::YAML => match serde_yaml::from_str(&manifest_content) {
                 Ok(m) => m,
                 Err(e) => {
-                    return Err(format!("Failed to parse the Flatpak manifest: {}.", e));
+                    return Err(format!(
+                        "Failed to parse the Flatpak application manifest: {}.",
+                        e
+                    ));
                 }
-            };
-            flatpak_manifest.format = FlatpakManifestFormat::YAML;
-        } else if crate::filename::is_json(&manifest_path) {
-            let json_content_without_comments = crate::utils::remove_comments_from_json(manifest_content);
-            flatpak_manifest = match serde_json::from_str(&json_content_without_comments) {
-                Ok(m) => m,
-                Err(e) => {
-                    return Err(format!("Failed to parse the Flatpak manifest: {}.", e));
+            },
+            FlatpakManifestFormat::JSON => {
+                let json_content_without_comments = crate::utils::remove_comments_from_json(manifest_content);
+                match serde_json::from_str(&json_content_without_comments) {
+                    Ok(m) => m,
+                    Err(e) => {
+                        return Err(format!(
+                            "Failed to parse the Flatpak application manifest: {}.",
+                            e
+                        ));
+                    }
                 }
-            };
-            flatpak_manifest.format = FlatpakManifestFormat::JSON;
-        }
+            }
+        };
+        flatpak_manifest.format = format;
 
         // From https://docs.flatpak.org/en/latest/manifests.html#basic-properties:
         // Each manifest file should specify basic information about the application that is to be built,
@@ -382,6 +384,7 @@ impl FlatpakApplication {
 #[derive(Serialize)]
 #[derive(Debug)]
 #[derive(Default)]
+#[derive(Hash)]
 #[serde(rename_all = "kebab-case")]
 #[serde(default)]
 /// Extension define extension points in the app/runtime that can be implemented by extensions,
@@ -473,18 +476,18 @@ mod tests {
     #[test]
     #[should_panic]
     pub fn test_parse_invalid_yaml() {
-        FlatpakApplication::parse("manifest.yaml", "----------------------------").unwrap();
+        FlatpakApplication::parse(FlatpakManifestFormat::YAML, "----------------------------").unwrap();
     }
 
     #[test]
     pub fn test_parse_missing_fields() {
         assert!(FlatpakApplication::parse(
-            "manifest.yaml",
+            FlatpakManifestFormat::YAML,
             r###"
             runtime: org.gnome.Platform
             runtime-version: "3.36"
             sdk: org.gnome.Sdk
-            command: flatpak-review
+            command: flatpak-rs
             "###
         )
         .is_err());
@@ -493,30 +496,30 @@ mod tests {
     #[test]
     pub fn test_parse() {
         match FlatpakApplication::parse(
-            "manifest.yaml",
+            FlatpakManifestFormat::YAML,
             r###"
-            app-id: net.louib.flatpak-review
+            app-id: net.louib.flatpak-rs
             runtime: org.gnome.Platform
             runtime-version: "3.36"
             sdk: org.gnome.Sdk
-            command: flatpak-review
+            command: flatpak-rs
             tags: ["nightly"]
             modules:
               -
-                name: "flatpak-review"
+                name: "flatpak-rs"
                 buildsystem: simple
                 cleanup: [ "*" ]
                 config-opts: []
                 sources:
                   -
                     type: git
-                    url: https://github.com/louib/flatpak-review.git
+                    url: https://github.com/louib/flatpak-rs.git
                     branch: master
             "###,
         ) {
-            Err(e) => panic!(e),
+            Err(e) => std::panic::panic_any(e),
             Ok(manifest) => {
-                assert_eq!(manifest.app_id, "net.louib.flatpak-review");
+                assert_eq!(manifest.app_id, "net.louib.flatpak-rs");
             }
         }
     }
@@ -524,32 +527,32 @@ mod tests {
     #[test]
     pub fn test_parse_shared_modules() {
         match FlatpakApplication::parse(
-            "manifest.yaml",
+            FlatpakManifestFormat::YAML,
             r###"
-            app-id: net.louib.flatpak-review
+            app-id: net.louib.flatpak-rs
             runtime: org.gnome.Platform
             runtime-version: "3.36"
             sdk: org.gnome.Sdk
-            command: flatpak-review
+            command: flatpak-rs
             tags: ["nightly"]
             modules:
               -
-                name: "flatpak-review"
+                name: "flatpak-rs"
                 buildsystem: simple
                 cleanup: [ "*" ]
                 config-opts: []
                 sources:
                   -
                     type: git
-                    url: https://github.com/louib/flatpak-review.git
+                    url: https://github.com/louib/flatpak-rs.git
                     branch: master
               -
                 "shared-modules/linux-audio/lv2.json"
             "###,
         ) {
-            Err(e) => panic!(e),
+            Err(e) => std::panic::panic_any(e),
             Ok(manifest) => {
-                assert_eq!(manifest.app_id, "net.louib.flatpak-review");
+                assert_eq!(manifest.app_id, "net.louib.flatpak-rs");
             }
         }
     }
@@ -557,7 +560,7 @@ mod tests {
     #[test]
     pub fn test_parse_add_extensions() {
         match FlatpakApplication::parse(
-            "manifest.yaml",
+            FlatpakManifestFormat::YAML,
             r###"
             app-id: net.pcsx2.PCSX2
             runtime: org.freedesktop.Platform
@@ -587,7 +590,7 @@ mod tests {
                     enable-if: "active-gl-driver"
             "###,
         ) {
-            Err(e) => panic!(e),
+            Err(e) => std::panic::panic_any(e),
             Ok(manifest) => {
                 assert_eq!(manifest.app_id, "net.pcsx2.PCSX2");
                 assert_eq!(manifest.add_extensions.len(), 3);
@@ -598,17 +601,17 @@ mod tests {
     #[test]
     pub fn test_parse_string_source() {
         match FlatpakApplication::parse(
-            "manifest.yaml",
+            FlatpakManifestFormat::YAML,
             r###"
-            app-id: net.louib.flatpak-review
+            app-id: net.louib.flatpak-rs
             runtime: org.gnome.Platform
             runtime-version: "3.36"
             sdk: org.gnome.Sdk
-            command: flatpak-review
+            command: flatpak-rs
             tags: ["nightly"]
             modules:
               -
-                name: "flatpak-review"
+                name: "flatpak-rs"
                 buildsystem: simple
                 cleanup: [ "*" ]
                 config-opts: []
@@ -617,9 +620,9 @@ mod tests {
                     "shared-modules/linux-audio/lv2.json"
             "###,
         ) {
-            Err(e) => panic!(e),
+            Err(e) => std::panic::panic_any(e),
             Ok(manifest) => {
-                assert_eq!(manifest.app_id, "net.louib.flatpak-review");
+                assert_eq!(manifest.app_id, "net.louib.flatpak-rs");
             }
         }
     }
@@ -627,13 +630,13 @@ mod tests {
     #[test]
     pub fn test_parse_source_without_type() {
         match FlatpakApplication::parse(
-            "manifest.yaml",
+            FlatpakManifestFormat::YAML,
             r###"
-            app-id: net.louib.flatpak-review
+            app-id: net.louib.flatpak-rs
             runtime: org.gnome.Platform
             runtime-version: "3.36"
             sdk: org.gnome.Sdk
-            command: flatpak-review
+            command: flatpak-rs
             tags: ["nightly"]
             modules:
               -
@@ -648,9 +651,9 @@ mod tests {
 
             "###,
         ) {
-            Err(e) => panic!(e),
+            Err(e) => std::panic::panic_any(e),
             Ok(manifest) => {
-                assert_eq!(manifest.app_id, "net.louib.flatpak-review");
+                assert_eq!(manifest.app_id, "net.louib.flatpak-rs");
             }
         }
     }
@@ -658,17 +661,17 @@ mod tests {
     #[test]
     pub fn test_parse_build_options() {
         match FlatpakApplication::parse(
-            "manifest.yaml",
+            FlatpakManifestFormat::YAML,
             r###"
-            app-id: net.louib.flatpak-review
+            app-id: net.louib.flatpak-rs
             runtime: org.gnome.Platform
             runtime-version: "3.36"
             sdk: org.gnome.Sdk
-            command: flatpak-review
+            command: flatpak-rs
             tags: ["nightly"]
             modules:
               -
-                name: "flatpak-review"
+                name: "flatpak-rs"
                 buildsystem: simple
                 cleanup: [ "*" ]
                 build-options:
@@ -685,9 +688,9 @@ mod tests {
                     "shared-modules/linux-audio/lv2.json"
             "###,
         ) {
-            Err(e) => panic!(e),
+            Err(e) => std::panic::panic_any(e),
             Ok(manifest) => {
-                assert_eq!(manifest.app_id, "net.louib.flatpak-review");
+                assert_eq!(manifest.app_id, "net.louib.flatpak-rs");
             }
         }
     }
@@ -695,17 +698,17 @@ mod tests {
     #[test]
     pub fn test_parse_script_source() {
         match FlatpakApplication::parse(
-            "manifest.yaml",
+            FlatpakManifestFormat::YAML,
             r###"
-            app-id: net.louib.flatpak-review
+            app-id: net.louib.flatpak-rs
             runtime: org.gnome.Platform
             runtime-version: "3.36"
             sdk: org.gnome.Sdk
-            command: flatpak-review
+            command: flatpak-rs
             tags: ["nightly"]
             modules:
               -
-                name: "flatpak-review"
+                name: "flatpak-rs"
                 buildsystem: simple
                 cleanup: [ "*" ]
                 config-opts: []
@@ -720,9 +723,9 @@ mod tests {
                         sed -i -e 's/\${\${NAME}_BIN}-NOTFOUND/\${NAME}_BIN-NOTFOUND/' cpp/CMakeLists.txt
             "###,
         ) {
-            Err(e) => panic!(e),
+            Err(e) => std::panic::panic_any(e),
             Ok(manifest) => {
-                assert_eq!(manifest.app_id, "net.louib.flatpak-review");
+                assert_eq!(manifest.app_id, "net.louib.flatpak-rs");
             }
         }
     }
@@ -730,7 +733,7 @@ mod tests {
     #[test]
     pub fn test_parse_json() {
         match FlatpakApplication::parse(
-            "manifest.json",
+            FlatpakManifestFormat::JSON,
             r###"
             {
                 "app-id": "org.gnome.SoundJuicer",
@@ -789,7 +792,7 @@ mod tests {
             }
             "###,
         ) {
-            Err(e) => panic!(e),
+            Err(e) => std::panic::panic_any(e),
             Ok(manifest) => {
                 assert_eq!(manifest.app_id, "org.gnome.SoundJuicer");
             }
@@ -799,7 +802,7 @@ mod tests {
     #[test]
     pub fn test_parse_json_with_comments() {
         match FlatpakApplication::parse(
-            "manifest.json",
+            FlatpakManifestFormat::JSON,
             r###"
             {
                 "app-id": "org.gnome.SoundJuicer",
@@ -876,7 +879,7 @@ mod tests {
             }
             "###,
         ) {
-            Err(e) => panic!(e),
+            Err(e) => std::panic::panic_any(e),
             Ok(manifest) => {
                 assert_eq!(manifest.app_id, "org.gnome.SoundJuicer");
             }
@@ -886,7 +889,7 @@ mod tests {
     #[test]
     pub fn test_parse_json_with_multi_line_comments() {
         match FlatpakApplication::parse(
-            "manifest.json",
+            FlatpakManifestFormat::JSON,
             r###"
             {
               "app-id": "org.gnome.Lollypop",
@@ -933,7 +936,7 @@ mod tests {
             }
             "###,
         ) {
-            Err(e) => panic!(e),
+            Err(e) => std::panic::panic_any(e),
             Ok(manifest) => {
                 assert_eq!(manifest.app_id, "org.gnome.Lollypop");
             }
@@ -943,7 +946,7 @@ mod tests {
     #[test]
     pub fn test_parse_extension() {
         match FlatpakApplication::parse(
-            "manifest.json",
+            FlatpakManifestFormat::JSON,
             r###"
             {
                 "id": "org.freedesktop.Platform.Icontheme.Paper",
@@ -963,7 +966,7 @@ mod tests {
             }
             "###,
         ) {
-            Err(e) => panic!(e),
+            Err(e) => std::panic::panic_any(e),
             Ok(manifest) => {
                 assert_eq!(manifest.id, "org.freedesktop.Platform.Icontheme.Paper");
                 assert_eq!(manifest.get_id(), "org.freedesktop.Platform.Icontheme.Paper");
@@ -974,7 +977,7 @@ mod tests {
     #[test]
     pub fn test_parse_recursive_modules() {
         match FlatpakApplication::parse(
-            "manifest.yaml",
+            FlatpakManifestFormat::YAML,
             r###"
             app-id: com.georgefb.haruna
             runtime: org.kde.Platform
@@ -1147,7 +1150,7 @@ mod tests {
                         sha256: 'd80023ab221b3cb89229b632e247035a22c5afaee9a7b3c653bbd702f71c1083'
             "###,
         ) {
-            Err(e) => panic!(e),
+            Err(e) => std::panic::panic_any(e),
             Ok(manifest) => {
                 assert_eq!(manifest.app_id, "com.georgefb.haruna");
                 assert_eq!(manifest.get_max_depth(), 3);
